"use client";
import React, { useEffect, useMemo, useState } from "react";
import { Strip, StripEvent } from "./component/Strip";
import {
  Button,
  Container,
  createTheme,
  CssBaseline,
  Divider,
  Slide,
  Snackbar,
  Stack,
  ThemeProvider,
  Typography,
  useMediaQuery,
} from "@mui/material";
import { Bus } from "./component/Bus";
import Grid from "@mui/material/Unstable_Grid2";

function random(min: number, max: number, floor: boolean = true) {
  const value = Math.random() * (max - min + 1) + min;
  return floor ? Math.floor(value) : value;
}

export default function Home() {
  const theme = useMemo(() => createTheme({ palette: { mode: "dark" } }), []);
  function onStripEvent(event: StripEvent) {
    console.log(event);
  }
  const isSnackBarHidden = useMemo(() => {
    if (typeof localStorage === "undefined") return false;
    return localStorage.getItem("snackBarHidden") === "true";
  }, []);
  const [snackBarVisible, setSnackBarVisibility] = useState(false);
  const breakPoints = useMemo(() => theme.breakpoints.values, []);
  const [width, setWidth] = useState(
    typeof document !== "undefined" ? window.innerWidth : 0
  );
  useEffect(() => {
    setSnackBarVisibility(!isSnackBarHidden);
  }, []);
  useEffect(() => {
    const values = {} as Record<string, [number, "bigger" | "smaller" | "equal"]>;
    type valuesKey = keyof typeof breakPoints;
    for (const key in breakPoints) {
      if (Object.prototype.hasOwnProperty.call(breakPoints, key)) {
        const breakPointWidth = breakPoints[key as keyof typeof breakPoints];
        values[key as valuesKey] =
          [breakPointWidth, breakPointWidth === width
            ? "equal"
            : breakPointWidth > width
            ? "bigger"
            : "smaller"];
      }
    }
    console.clear()
    console.log(
      width,
      `Closest: ${
        Object.entries(breakPoints).reduce((acc, i) => {
          const equation = Math.abs(i[1] - width) <= Math.abs(acc[1] - width) ? i : acc;
          return equation;
        })[0]
      }`,
    );
    console.table(values);
  }, [width]);
  const strips = { virtual: 3, physical: 5 };
  const buses = { virtual: 3, physical: 5 };
  const amountOfStrips = strips.physical + strips.virtual;
  const amountOfBuses = buses.physical + buses.virtual;
  let clicks = 0;
  let timeout: NodeJS.Timeout | undefined;
  if (typeof document !== "undefined") {
    document.onclick = (e) => {
      clicks++;
      if (timeout !== undefined) {
        clearTimeout(timeout);
        timeout = undefined;
      }
      timeout = setTimeout(() => (clicks = 0), 250);
      console.log(e);
      if (clicks < 2) return;
      clearTimeout(timeout);
      clicks = 0;

      // e.stopPropagation();

      // if (document.fullscreenElement !== null) document.exitFullscreen();
      // else document.body.requestFullscreen({ navigationUI: "hide" });
    };
    window.onresize = () => {
      setWidth(window.innerWidth);
    };
  }

  const snackBarActionButton = (
    <>
      <Button
        onClick={() => {
          localStorage.setItem("snackBarHidden", "true");
          setSnackBarVisibility(false);
        }}
        color="secondary"
      >
        Gotcha
      </Button>
    </>
  );
  return (
    <ThemeProvider theme={theme}>
      <Container maxWidth={"xl"} sx={{ height: "100%", paddingInline: "5px" }}>
        <div>
          <CssBaseline />
          <Typography variant="h5">Inputs</Typography>
          <Grid
            container
            spacing={width ? 0 : 1}
            display={"flex"}
            alignItems={"center"}
            justifyContent={"space-evenly"}
            columns={{ xs: 8, sm: 12, md: 5, lg: 12 }}
          >
            {[...Array(amountOfStrips)].map((_v, stripId) => (
              <React.Fragment
                key={`${stripId < amountOfStrips ? "a" : "b"}${
                  stripId < amountOfStrips
                    ? stripId + 1
                    : stripId + 1 - amountOfStrips
                }`}
              >
                {stripId === amountOfStrips && width > breakPoints.lg && (
                  <Grid>
                    <Divider
                      sx={{ marginBlock: "15px", borderWidth: "1px" }}
                      variant="fullWidth"
                      orientation="vertical"
                    >
                      Virtual Inputs
                    </Divider>
                  </Grid>
                )}
                <Grid
                  lg={stripId + 1 === strips.physical ? 2 : 0}
                  xs={1}
                  sm={1}
                  md={0}
                  sx={{ minWidth: "fit-content" }}
                >
                  <Typography variant="overline">{`Strip #${
                    stripId + 1
                  }`}</Typography>
                  <Strip
                    onChange={(ev) => onStripEvent(ev)}
                    physicalBuses={buses.physical}
                    virtualBuses={buses.virtual}
                  />
                </Grid>
              </React.Fragment>
            ))}
          </Grid>

          <Typography variant="h5">Outputs</Typography>
          <Grid container spacing={1} direction={"column"}>
            {[...Array(amountOfBuses)].map((_v, busId) => (
              <Grid
                key={`${busId < buses.physical ? "a" : "b"}${
                  busId < buses.physical
                    ? busId + 1
                    : busId + 1 - buses.physical
                }`}
                xs={12}
              >
                <Typography variant="overline">{`Bus ${
                  busId < buses.physical ? "a" : "b"
                }${
                  busId < buses.physical
                    ? busId + 1
                    : busId + 1 - buses.physical
                }`}</Typography>
                <Bus onChange={console.log} />
              </Grid>
            ))}
          </Grid>
        </div>
      </Container>
      <Snackbar
        anchorOrigin={{ horizontal: "center", vertical: "top" }}
        TransitionComponent={Slide}
        transitionDuration={250}
        open={snackBarVisible}
        message="Triple tap on empty space for toggle fullscreen"
        action={snackBarActionButton}
      />
    </ThemeProvider>
  );
}
